/* File: 1000 HwrBacklightPat.c
   ============================
   This file contains a patch for the trap "HwrBacklight".  All it does it invert
   the pixel polarity and then pass control over to the standard HwrBacklight function.
   
   Written by Douglas I. Anderson (dianders@cs.stanford.edu)

   Hack patch info
   ---------------
   code id:   1000
   atrap:	  A2EA (sysTrapHwrBacklight)

   History
   -------
   03/23/99 Created
   03/23/99 Added always/usually/sometimes modes.
   06/06/99 Changed so that turning off the backlight always turns invert off.
 */

// ----------------------------------
// Interfaces...
// ----------------------------------

/* System-wide */
#include <Pilot.h>
#include <Graffiti.h>

/* Hack specific */
#include "GreenLightHack.h"

// ----------------------------------
// Local types...
// ----------------------------------

typedef Boolean (*HwrBacklightFn) (Boolean set, Boolean newState);

// ----------------------------------
// Local function prototypes...
// ----------------------------------

// ----------------------------------
// Trap Handler...
// ----------------------------------

/* Function: MyHwrBacklight()
   --------------------------
   This is the entry function of this patch.  All system calls to HwrBacklight
   will be redirected here.  Basically, if the caller is actually using this
   function to set the backlight backlight (set is true), then we want to flip the
   bits.  However, sometimes PalmOS calls this to set an already off backlight to
   be off.  Thus, we check the old state and only flip the bit if the state is changing.
   
   This function has three modes that are chosen at compile time.  You can either have
   it always invert, invert usually (when the punctuation mode isn't there), or invert
   sometimes (when the punctuation mode is there).  Set the mode using the #define at
   the top of the function.  Compile-time choosing was used because it keeps the hack
   small, I didn't have to write a config dialog, and this is not something you usually
   change on a daily basis.
   
   Note: I figured out the meanings of the parameters and the return by trial and
   error.  Hopefully they're all correct.  :)
   
   @param set      Tells whether to set the backlight or just return the current
   				   value.
   @param newState Tells what value to set the backlight to.
   @return         The value of the backlight when the function is done.
 */

#define ALWAYS 		0
#define USUALLY 	1
#define SOMETIMES 	2
#define MODE 		ALWAYS

Boolean	MyHwrBacklight (Boolean set, Boolean newState);
Boolean	MyHwrBacklight (Boolean set, Boolean newState)
{
	HwrBacklightFn oldTrap;
	DWord temp;

	FtrGet (kCreatorCode, kHwrBacklightPatchID, &temp);
	oldTrap = (HwrBacklightFn)temp;

	if (set) {
#if (MODE == USUALLY)
		Boolean bogus;
		Word tempShift;
		GrfGetState(&bogus, &bogus, &tempShift, &bogus);
		if (tempShift != grfTempShiftPunctuation || !newState)
#elif (MODE == SOMETIMES)
		Boolean bogus;
		Word tempShift;
		GrfGetState(&bogus, &bogus, &tempShift, &bogus);
		if (tempShift == grfTempShiftPunctuation || !newState)
#endif // MODE (usually vs sometimes)
		{
			const char kPixPolMask = 0x01;
			char * const LPOLCF = (char *)0xFFFFFA21;

			if (newState) {
				// Enable if turning on...
				(*LPOLCF) |= kPixPolMask;
			} else {
				// Disable if turning off...
				(*LPOLCF) &= ~kPixPolMask;
			}
		}
	}
	return oldTrap (set, newState);
}











